@@ -1012,7 +1012,7 @@ class ElementPath(object):
         if context is None:
             raise ValidationError(
                 "'{0}' is not valid path for FHIR Release '{1}'".format(
-                    self._raw, fhir_release.value
+                    self._raw, fhir_release.name
                 )
             )
 
